
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#pragma once

#if !defined(__COGL_H_INSIDE__) && !defined(COGL_COMPILATION)
#error "Only <cogl/cogl.h> can be included directly."
#endif

#include "cogl/cogl-types.h"

G_BEGIN_DECLS
/* enumerations from "cogl-buffer.h" */
COGL_EXPORT GType cogl_buffer_update_hint_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_BUFFER_UPDATE_HINT (cogl_buffer_update_hint_get_type())
COGL_EXPORT GType cogl_buffer_access_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_BUFFER_ACCESS (cogl_buffer_access_get_type())
COGL_EXPORT GType cogl_buffer_map_hint_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_BUFFER_MAP_HINT (cogl_buffer_map_hint_get_type())
COGL_EXPORT GType cogl_buffer_bind_target_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_BUFFER_BIND_TARGET (cogl_buffer_bind_target_get_type())
/* enumerations from "cogl-pixel-format.h" */
COGL_EXPORT GType cogl_pixel_format_get_type (void) G_GNUC_CONST;
#define COGL_TYPE_PIXEL_FORMAT (cogl_pixel_format_get_type())
G_END_DECLS

/* Generated data ends here */

